@@ -63,17 +63,18 @@ sub accept_rules {
   my ( $target, $src, $src_if, $dst, $dst_if, $options ) = @_;
 
   my ($snmp,$trap) = $self->prototypes( $target, $options );
+  my $masq = defined $options->{portfw} ? PORTFW :
+    $options->{masq} ? MASQ : NOMASQ;
+
   if ( ipv4_in_network( $src, $dst ) ) {
     accept_ip_ruleset( $snmp, $src, $src_if, $src_if->{broadcast}, $dst_if,
-		       $options->{masq} ? MASQ : NOMASQ
-		     );
+		       $masq, $options->{portfw} );
   }
   accept_udp_ruleset( $snmp, $src, $src_if, $dst, $dst_if,
-		     $options->{masq} ? MASQ : NOMASQ
-		   );
+		      $masq, $options->{portfw} );
+
   accept_ip_ruleset( $trap, $dst, $dst_if, $src, $src_if,
-		    $options->{masq} ? MASQ : NOMASQ
-		  );
+		     $masq, $options->{portfw} );
 }
 
 sub account_rules {
@@ -81,17 +82,17 @@ sub account_rules {
   my ( $target, $src, $src_if, $dst, $dst_if, $options ) = @_;
 
   my ($snmp,$trap) = $self->prototypes( $target, $options );
+  my $masq = defined $options->{portfw} ? PORTFW :
+    $options->{masq} ? MASQ : NOMASQ;
+
   if ( ipv4_in_network( $src, $dst ) ) {
     acct_ip_ruleset( $snmp, $src, $src_if, $src_if->{broadcast}, $dst_if,
-		     $options->{masq} ? MASQ : NOMASQ
-		   );
+		      $masq, $options->{portfw} );
   }
   acct_udp_ruleset( $snmp, $src, $src_if, $dst, $dst_if,
-		     $options->{masq} ? MASQ : NOMASQ
-		   );
+		      $masq, $options->{portfw} );
   acct_ip_ruleset( $trap, $dst, $dst_if, $src, $src_if,
-		   $options->{masq} ? MASQ : NOMASQ
-		   );
+		      $masq, $options->{portfw} );
 }
 
 sub valid_options {
@@ -100,6 +101,7 @@ sub valid_options {
 }
 
 1;
+
 =pod
 
 =head1 NAME
